/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.model;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.audio.AudioTrackComparator;
import com.googlepages.dronten.jtagger.util.Log;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;


/**
 * TrackModel contains all mp3 files from the current selected directory.
 */
public class TrackModel extends AbstractTableModel {
    private static final long       serialVersionUID = 1;
    private MouseHandler            aMouseHandler;
    private AudioTrackComparator    aComparator = new AudioTrackComparator();
    private boolean                 aSortAscending = true;
    private ArrayList<AudioTrack>   aAudioFiles = new ArrayList<AudioTrack>();
    private ArrayList<AudioTrack>   aTestAudioFiles = new ArrayList<AudioTrack>();


    /**
     * Singleton object.
     */
    public TrackModel() {
        aMouseHandler = new MouseHandler();
    }


    /**
     * Add new mp3 file to the model.
     *
     * @param file - An MP3 object representing an mp3 song
     */
    public void add(AudioTrack file) {
        aAudioFiles.add(file);
    }


    /**
     * Clear all songs
     */
    public void clear() {
        aAudioFiles.clear();
    }


    /**
     * Clear all test model
     */
    public void clearTest() {
        aTestAudioFiles.clear();
    }


    /**
     * Get file array.
     *
     * @return Array of tracks
     */
    public ArrayList<AudioTrack> getAudioTracks() {
        return aAudioFiles;
    }


    /**
     * Return what type of object table is going to edit.
     *
     * @param column
     * @return Class of table cell
     */
    @Override public Class getColumnClass(int column) {
        switch (column) {
            case 0:
                return Integer.class;

            default:
                return String.class;
        }
    }


    /**
     * Return number of columns.
     *
     * @return Columns
     */
    @Override public int getColumnCount() {
        return 3;
    }


    /**
     * Get table column name.
     *
     * @param col - Column number
     * @return Name of column
     */
    @Override public String getColumnName(int col) {
        if (col == 0) {
            return "Track";
        }
        else if (col == 1) {
            return "File";
        }
        else {
            return "Time";
        }
    }


    /**
     * Get track comparator object.
     *
     * @return Comparator object
     */
    public AudioTrackComparator getComparator() {
        return aComparator;
    }


    /**
     * Get mouse handler.
     *
     * @return Mouse handler
     */
    public MouseHandler getMouseHandler() {
        return aMouseHandler;
    }


    /**
     * Return number of rows.
     *
     * @return Number of rows
     */
    @Override public int getRowCount() {
        return aAudioFiles.size();
    }


    /**
     * Get test files array.<br>
     *  This array is created after a test is done but no real changes has been done to the files.
     *
     * @return Array of tracks that has had a test on them
     */
    public ArrayList<AudioTrack> getTestAudioTracks() {
        return aTestAudioFiles;
    }


    /**
     * Return MP3 test object.
     *
     * @param row   Row to return MP3 for
     * @return      MP3 object or null
     */
    public AudioTrack getTestTrack(int row) {
        if (aTestAudioFiles != null && row < aTestAudioFiles.size()) {
            return aTestAudioFiles.get(row);
        }
        else {
            return null;
        }
    }


    /**
     * Return MP3 object.
     *
     * @param row   Row to return MP3 for
     * @return      MP3 object or null
     */
    public AudioTrack getTrack(int row) {
        if (aAudioFiles != null && row < aAudioFiles.size()) {
            return aAudioFiles.get(row);
        }
        else {
            return null;
        }
    }


    /**
     * Return either filename or the time as string.
     *
     * @param row
     * @param col
     * @return Object to paint
     */
    @Override public Object getValueAt(int row, int col) {
        AudioTrack track = getTrack(row);
        AudioTrack track_ = JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTestTrack(row);

        try {
            if (track != null) {
                if (col == 0) {
                    if (track_ != null) {
                        return track_.aTrack;
                    }
                    else {
                        return track.aTrack;
                    }
                }
                else if (col == 1) {
                    return track.aFileName;
                }
                else {
                    return track.aTime;
                }
            }
            else {
                return "";
            }
        }
        catch (Exception e) {
            return "";
        }
    }


    /**
     * Only filename can be edited.
     *
     * @param row
     * @param col
     * @return true if cell is editable
     */
    @Override public boolean isCellEditable(int row, int col) {
        switch (col) {
            case 0:
            case 1:
                return true;

            default:
                return false;
        }

    }


    /**
     * Get sort order, up or down.
     *
     * @return true for sorting in ascending order
     */
    public boolean isSortAscending() {
        return aSortAscending;
    }


    /**
     * Set sorting order.
     *
     * @param sortAscending true for sorting in ascending order
     */
    public void setSortAscending(boolean sortAscending) {
        this.aSortAscending = sortAscending;
    }


    /**
     * BaseRename file.
     *
     * @param value
     * @param row
     * @param col
     */
    @Override public void setValueAt(Object value, int row, int col) {
        AudioTrack track = getTrack(row);

        try {
            if (track != null) {
                if (col == 0) {
                    Log.get().addTime(2, String.format("Changing track for file '%s' from '%s' to '%s'", track.aFileName, track.aTrack, value.toString()));
                    track.aTrack = value.toString();
                    track.write(track.aType);
                }
                else if (col == 1) {
                    Log.get().addTime(2, String.format("Changing filename for %s to %s", track.aFileName, (String) value));
                    track.rename((String) value);
                }
            }

            JTagger.get().setMessage(Constants.MESSAGE_WRITE_OK, false);
            JTagger.get().reloadAllTables();
        }
        catch (Exception e) {
            JTagger.get().setMessage(e.getMessage(), true);
            JTagger.get().reloadAllTables();
        }
    }


    /**
     * Move MP3 song down in list.
     *
     * @param row
     */
    public void moveDown(int row) {
        if (row < aAudioFiles.size() - 1) {
            AudioTrack track = aAudioFiles.get(row + 1);
            aAudioFiles.set(row + 1, aAudioFiles.get(row));
            aAudioFiles.set(row, track);
            JTagger.get().selectRow(row + 1);
        }
    }


    /**
     * Move MP3 song up in list.
     *
     * @param row
     */
    public void moveUp(int row) {
        if (row > 0) {
            AudioTrack track = aAudioFiles.get(row - 1);
            aAudioFiles.set(row - 1, aAudioFiles.get(row));
            aAudioFiles.set(row, track);
            JTagger.get().selectRow(row - 1);
        }
    }


    /**
     * Sort MP3 files. Use the MP3Comparator object for comparing files.
     */
    public void sort() {
        if (aSortAscending) {
            Collections.sort(aAudioFiles, aComparator);
        }
        else {
            Collections.sort(aAudioFiles, aComparator);
            Collections.reverse(aAudioFiles);
        }

        JTagger.get().reloadAllTables();
    }


    /**
     * Sort files when user is clicking the table header.
     */
    private class MouseHandler extends MouseAdapter {
        /**
         *
         * @param mouseEvent
         */
        @Override public void mouseClicked(MouseEvent mouseEvent) {
            JTableHeader        header = (JTableHeader) mouseEvent.getSource();
            TableColumnModel    columnModel = header.getColumnModel();
            int                 viewColumn = columnModel.getColumnIndexAtX(mouseEvent.getX());
            int                 column = columnModel.getColumn(viewColumn).getModelIndex();

            if (column != -1) {
                if (column == 0) {
                    aComparator.aSortType = AudioTrackComparator.SORT.COMPARE_TRACK;
                }
                else if (column == 1) {
                    aComparator.aSortType = AudioTrackComparator.SORT.COMPARE_FILE;
                }
                else {
                    aComparator.aSortType = AudioTrackComparator.SORT.COMPARE_TIME;
                }

                if (mouseEvent.isControlDown()) {
                    aSortAscending = false;
                    sort();
                }
                else {
                    aSortAscending = true;
                    sort();
                }
            }
        }
    }


    /**
     * Start displaying model from test mp3 objects.
     */
    public void startTest() {
        clearTest();

        for (AudioTrack track : aAudioFiles) {
            try {
                aTestAudioFiles.add((AudioTrack) track.clone());
            }
            catch (CloneNotSupportedException e) {
            }
        }
    }
}
